@@ -31,9 +31,8 @@
 #include "common/gkr-buffer.h"
 #include "common/gkr-location.h"
 
-#include "daemon/gnome-keyring-daemon.h"
-
 #include "library/gnome-keyring-memory.h"
+#include "library/gnome-keyring-private.h"
 #include "library/gnome-keyring-proto.h"
 
 #include <glib.h>
@@ -301,14 +300,14 @@ generate_acl_data (GkrBuffer *buffer,
 		ac = l->data;
 		
 		gkr_buffer_add_uint32 (buffer, ac->types_allowed);
-		if (!gnome_keyring_proto_add_utf8_string (buffer, ac->application->display_name)) {
+		if (!gkr_proto_add_utf8_string (buffer, ac->application->display_name)) {
 			return FALSE;
 		}
-		if (!gnome_keyring_proto_add_utf8_string (buffer, ac->application->pathname)) {
+		if (!gkr_proto_add_utf8_string (buffer, ac->application->pathname)) {
 			return FALSE;
 		}
 		/* Reserved: */
-		if (!gnome_keyring_proto_add_utf8_string (buffer, NULL)) {
+		if (!gkr_proto_add_utf8_string (buffer, NULL)) {
 			return FALSE;
 		}
 		gkr_buffer_add_uint32 (buffer, 0);
@@ -326,28 +325,28 @@ generate_encrypted_data (GkrBuffer *buffer, GkrKeyring *keyring)
 	GkrKeyringItem *item;
 	
 	/* Make sure we're using non-pageable memory */
-	gnome_keyring_proto_go_secure (buffer);
+	gkr_proto_go_secure (buffer);
 	
 	for (l = keyring->items; l != NULL; l = l->next) {
 		item = l->data;
-		if (!gnome_keyring_proto_add_utf8_string (buffer, item->display_name)) {
+		if (!gkr_proto_add_utf8_string (buffer, item->display_name)) {
 			return FALSE;
 		}
-		if (!gnome_keyring_proto_add_utf8_secret (buffer, item->secret)) {
+		if (!gkr_proto_add_utf8_secret (buffer, item->secret)) {
 			return FALSE;
 		}
-		gnome_keyring_proto_add_time (buffer, item->ctime);
-		gnome_keyring_proto_add_time (buffer, item->mtime);
+		gkr_proto_add_time (buffer, item->ctime);
+		gkr_proto_add_time (buffer, item->mtime);
 
 		/* reserved: */
-		if (!gnome_keyring_proto_add_utf8_string (buffer, NULL)) {
+		if (!gkr_proto_add_utf8_string (buffer, NULL)) {
 			return FALSE;
 		}
 		for (i = 0; i < 4; i++) {
 			gkr_buffer_add_uint32 (buffer, 0);
 		}
 
-		if (!gnome_keyring_proto_add_attribute_list (buffer, item->attributes)) {
+		if (!gkr_proto_add_attribute_list (buffer, item->attributes)) {
 			return FALSE;
 		}
 		if (!generate_acl_data (buffer, item->acl)) {
@@ -379,12 +378,12 @@ generate_file (GkrBuffer *buffer, GkrKeyring *keyring)
 	gkr_buffer_add_byte (buffer, 0); /* crypto (0 == AEL) */
 	gkr_buffer_add_byte (buffer, 0); /* hash (0 == MD5) */
 
-	if (!gnome_keyring_proto_add_utf8_string (buffer, keyring->keyring_name)) {
+	if (!gkr_proto_add_utf8_string (buffer, keyring->keyring_name)) {
 		return FALSE;
 	}
 
-	gnome_keyring_proto_add_time (buffer, keyring->mtime);
-	gnome_keyring_proto_add_time (buffer, keyring->ctime);
+	gkr_proto_add_time (buffer, keyring->mtime);
+	gkr_proto_add_time (buffer, keyring->ctime);
 	
 	flags = 0;
 	if (keyring->lock_on_idle) {
@@ -408,9 +407,9 @@ generate_file (GkrBuffer *buffer, GkrKeyring *keyring)
 		gkr_buffer_add_uint32 (buffer, item->id);
 		gkr_buffer_add_uint32 (buffer, item->type);
 		
-		hashed = gnome_keyring_attributes_hash (item->attributes);
+		hashed = gkr_keyring_item_attributes_hash (item->attributes);
 
-		if (!gnome_keyring_proto_add_attribute_list (buffer, hashed)) {
+		if (!gkr_proto_add_attribute_list (buffer, hashed)) {
 			gnome_keyring_attribute_list_free (hashed);
 			return FALSE;
 		}
@@ -436,6 +435,9 @@ generate_file (GkrBuffer *buffer, GkrKeyring *keyring)
 			     (guchar*)to_encrypt.buf + 16, to_encrypt.len - 16);
 	memcpy (to_encrypt.buf, digest, 16);
 	
+	/* This is either set by gnome_keyring_create, or when reading from disk */
+	g_assert (keyring->hash_iterations);
+	
 	if (!encrypt_buffer (&to_encrypt, keyring->password, keyring->salt, keyring->hash_iterations)) {
 		gkr_buffer_uninit (&to_encrypt);
 		return FALSE;
@@ -466,17 +468,14 @@ decode_acl (GkrBuffer *buffer, gsize offset, gsize *offset_out, GList **out)
 		if (!gkr_buffer_get_uint32 (buffer, offset, &offset, &x)) {
 			goto bail;
 		}
-		if (!gnome_keyring_proto_get_utf8_string (buffer, offset, &offset,
-							  &name)) {
+		if (!gkr_proto_get_utf8_string (buffer, offset, &offset, &name)) {
 			goto bail;
 		}
-		if (!gnome_keyring_proto_get_utf8_string (buffer, offset, &offset,
-							  &path)) {
+		if (!gkr_proto_get_utf8_string (buffer, offset, &offset, &path)) {
 			g_free (name);
 			goto bail;
 		}
-		if (!gnome_keyring_proto_get_utf8_string (buffer, offset, &offset,
-							  &reserved) ||
+		if (!gkr_proto_get_utf8_string (buffer, offset, &offset, &reserved) ||
 		    reserved != NULL) {
 			g_free (name);
 			g_free (path);
@@ -544,6 +543,7 @@ update_keyring_from_data (GkrKeyring *keyring, GkrBuffer *buffer)
 	GHashTable *checks = NULL;
 	GkrKeyringItem *item;
 	char *reserved;
+	gboolean ret = FALSE;
 
 	display_name = NULL;
 	items = 0;
@@ -569,14 +569,13 @@ update_keyring_from_data (GkrKeyring *keyring, GkrBuffer *buffer)
 		return FALSE;
 	}
 
-	if (!gnome_keyring_proto_get_utf8_string (buffer, offset, &offset,
-						  &display_name)) {
+	if (!gkr_proto_get_utf8_string (buffer, offset, &offset, &display_name)) {
 		goto bail;
 	}
-	if (!gnome_keyring_proto_get_time (buffer, offset, &offset, &ctime)) {
+	if (!gkr_proto_get_time (buffer, offset, &offset, &ctime)) {
 		goto bail;
 	}
-	if (!gnome_keyring_proto_get_time (buffer, offset, &offset, &mtime)) {
+	if (!gkr_proto_get_time (buffer, offset, &offset, &mtime)) {
 		goto bail;
 	}
 	if (!gkr_buffer_get_uint32 (buffer, offset, &offset, &flags)) {
@@ -588,7 +587,7 @@ update_keyring_from_data (GkrKeyring *keyring, GkrBuffer *buffer)
 	if (!gkr_buffer_get_uint32 (buffer, offset, &offset, &hash_iterations)) {
 		goto bail;
 	}
-	if (!gnome_keyring_proto_get_bytes (buffer, offset, &offset, salt, 8)) {
+	if (!gkr_proto_get_bytes (buffer, offset, &offset, salt, 8)) {
 		goto bail;
 	}
 	
@@ -616,8 +615,8 @@ update_keyring_from_data (GkrKeyring *keyring, GkrBuffer *buffer)
 						     &items[i].type)) {
 			goto bail;
 		}
-		if (!gnome_keyring_proto_decode_attribute_list (buffer, offset, &offset,
-								&items[i].hashed_attributes)) {
+		if (!gkr_proto_decode_attribute_list (buffer, offset, &offset,
+		                                      &items[i].hashed_attributes)) {
 			goto bail;
 		}
 	}
@@ -648,25 +647,25 @@ update_keyring_from_data (GkrKeyring *keyring, GkrBuffer *buffer)
 			locked = FALSE;
 			offset += 16; /* Skip hash */
 			for (i = 0; i < num_items; i++) {
-				if (!gnome_keyring_proto_get_utf8_string (buffer, offset, &offset,
-									  &items[i].display_name)) {
+				if (!gkr_proto_get_utf8_string (buffer, offset, &offset,
+				                                &items[i].display_name)) {
 					goto bail;
 				}
-				if (!gnome_keyring_proto_get_utf8_secret (buffer, offset, &offset,
-									  &items[i].secret)) {
+				if (!gkr_proto_get_utf8_secret (buffer, offset, &offset,
+				                                &items[i].secret)) {
 					goto bail;
 				}
-				if (!gnome_keyring_proto_get_time (buffer, offset, &offset,
-								   &items[i].ctime)) {
+				if (!gkr_proto_get_time (buffer, offset, &offset,
+				                         &items[i].ctime)) {
 					goto bail;
 				}
-				if (!gnome_keyring_proto_get_time (buffer, offset, &offset,
-								   &items[i].mtime)) {
+				if (!gkr_proto_get_time (buffer, offset, &offset,
+				                         &items[i].mtime)) {
 					goto bail;
 				}
 				reserved = NULL;
-				if (!gnome_keyring_proto_get_utf8_string (buffer, offset, &offset,
-									  &reserved) ||
+				if (!gkr_proto_get_utf8_string (buffer, offset, &offset,
+				                                &reserved) ||
 				    reserved != NULL) {
 					g_free (reserved);
 					goto bail;
@@ -681,8 +680,8 @@ update_keyring_from_data (GkrKeyring *keyring, GkrBuffer *buffer)
 						goto bail;
 					}
 				}
-				if (!gnome_keyring_proto_decode_attribute_list (buffer, offset, &offset,
-										&items[i].attributes)) {
+				if (!gkr_proto_decode_attribute_list (buffer, offset, &offset,
+				                                      &items[i].attributes)) {
 					goto bail;
 				}
 				
@@ -700,6 +699,7 @@ update_keyring_from_data (GkrKeyring *keyring, GkrBuffer *buffer)
 	keyring->locked = locked;
 	g_free (keyring->keyring_name);
 	keyring->keyring_name = display_name;
+	display_name = NULL;
 	keyring->mtime = mtime;
 	keyring->ctime = ctime;
 	keyring->lock_on_idle = !!(flags & LOCK_ON_IDLE_FLAG);
@@ -755,16 +755,16 @@ update_keyring_from_data (GkrKeyring *keyring, GkrBuffer *buffer)
 	}
 	
 	g_hash_table_foreach (checks, (GHFunc)remove_unavailable_item, keyring);
-	g_hash_table_destroy (checks);
 
-	return TRUE;
+	ret = TRUE;
+
  bail:
 	gkr_buffer_uninit (&to_decrypt);
 	if (checks)
 		g_hash_table_destroy (checks);
 	g_free (display_name);
 
-	if (items != NULL) {
+	if (items != NULL && !ret) {
 		for (i = 0; i < num_items; i++) {
 			g_free (items[i].display_name);
 			gnome_keyring_free_password (items[i].secret);
@@ -772,10 +772,10 @@ update_keyring_from_data (GkrKeyring *keyring, GkrBuffer *buffer)
 			gnome_keyring_attribute_list_free (items[i].attributes);
 			gnome_keyring_acl_free (items[i].acl);
 		}
-		g_free (items);
 	}
+	g_free (items);
 	
-	return FALSE;
+	return ret;
 }
 
 
@@ -807,6 +807,9 @@ get_default_location_for_name (GQuark base_loc, const char *keyring_name)
 	gchar *base, *filename;
 	int version;
 	GQuark loc;
+	
+	g_assert (base_loc);
+	g_assert (keyring_name && keyring_name[0]);
 
 	base = g_filename_from_utf8 (keyring_name, -1, NULL, NULL, NULL);
 	if (base == NULL)
@@ -851,9 +854,6 @@ gkr_keyring_init (GkrKeyring *keyring)
 	/* Default values: */
 	keyring->lock_on_idle = FALSE;
 	keyring->lock_timeout = 0;
-
-	keyring->hash_iterations = 1000 + (int) (1000.0 * rand() / (RAND_MAX + 1.0));
-	init_salt (keyring->salt);
 }
 
 static void
@@ -958,12 +958,20 @@ GkrKeyring*
 gkr_keyring_create (GQuark base_loc, const gchar *keyring_name, const gchar *password)
 {
 	GkrKeyring *keyring;
+
+	g_return_val_if_fail (keyring_name && keyring_name[0], NULL);
+	g_return_val_if_fail (password, NULL);
+		
+	if (!base_loc)
+		base_loc = GKR_LOCATION_BASE_LOCAL;
 	
 	keyring = gkr_keyring_new (keyring_name, 0);
 	if (keyring != NULL) {
 		keyring->location = get_default_location_for_name (base_loc, keyring_name);
 		keyring->locked = FALSE;
 		keyring->password = gnome_keyring_memory_strdup (password);
+		keyring->hash_iterations = 1000 + (int) (1000.0 * rand() / (RAND_MAX + 1.0));
+		init_salt (keyring->salt);
 		gkr_keyring_save_to_disk (keyring);
 	}
 	return keyring;
